package com.me.common.ecs;

public final class Entity {

    private static int nextId = 0;
    private static long nextUniqueId = 0;

    private Engine engine;
    protected final int id;

    protected long componentBits;
    protected long systemEnabledBits;

    protected boolean active;
    protected boolean removed;
    protected boolean pendingRefresh;

    private long uniqueId;

    protected Entity(Engine engine) {
        this.engine = engine;
        this.id = nextId++;
    }

    public int getId() {
        return id;
    }

    public long getUniqueId() {
        return uniqueId;
    }

    public boolean isActive() {
        return active;
    }

    public boolean isRemoved() {
        return removed;
    }

    public void activate() {
        active = true;
        refresh();
    }

    public void deactivate() {
        active = false;
        refresh();
    }

    public void remove() {
        removed = true;
        active = false;
        refresh();
    }

    public Engine getEngine() {
        return this.engine;
    }

    public boolean hasComponent(Class<? extends Component> clazz) {
        return ComponentType.getComponentType(clazz).isTypeInMask(componentBits);
    }

    public <T extends Component> T getComponent(Class<T> clazz) {
        return engine.getEntityComponent(this, clazz);
    }

    public void addComponent(Component component) {
        engine.addEntityComponent(this, component);
        refresh();
    }

    public void removeComponent(Component component) {
        engine.removeEntityComponent(this, component);
        refresh();
    }

    protected void addComponentType(ComponentType type) {
        componentBits |= type.bits;
    }

    protected void removeComponentType(ComponentType type) {
        componentBits &= ~type.bits;
    }

    protected void addSystemEnabledBit(long bit) {
        systemEnabledBits |= bit;
    }

    protected void removeSystemEnabledBit(long bit) {
        systemEnabledBits &= ~bit;
    }

    public void refresh() {
        if (!pendingRefresh) {
            engine.refreshEntity(this);
            pendingRefresh = true;
        }
    }

    protected void reset() {
        componentBits = 0;
        systemEnabledBits = 0;
        active = false;
        removed = false;
        pendingRefresh = false;
    }

    protected void updateUniqueId() {
        this.uniqueId = nextUniqueId++;
    }

    public String toString() {
        return "Entity[id=" + id + ", uniqueId=" + uniqueId + "]";
    }

}
